@@ -22,7 +22,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+# include "config.h"
 #endif
 
 #include <stdlib.h>
@@ -37,9 +37,9 @@ extern int errno;
 #include "xalloc.h"
 #include "xvasprintf.h"
 
-#include "os_env.h"
-#include "smtp.h"
 #include "list.h"
+#include "smtp.h"
+#include "tools.h"
 #include "conf.h"
 
 /* buffer size for configuration file lines */
@@ -75,11 +75,12 @@ account_t *account_new(const char *conffile, const char *id)
     a->password = NULL;
     a->ntlmdomain = NULL;
     a->tls = 0;
+    a->tls_nostarttls = 0;
     a->tls_key_file = NULL;
     a->tls_cert_file = NULL;
     a->tls_trust_file = NULL;
-    a->tls_nostarttls = 0;
     a->tls_nocertcheck = 0;
+    a->tls_force_sslv3 = 0;
     a->logfile = NULL;
     a->syslog = NULL;
     return a;
@@ -118,13 +119,14 @@ account_t *account_copy(account_t *acc)
 	a->password = acc->password ? xstrdup(acc->password) : NULL;
 	a->ntlmdomain = acc->ntlmdomain ? xstrdup(acc->ntlmdomain) : NULL;
 	a->tls = acc->tls;
+	a->tls_nostarttls = acc->tls_nostarttls;
 	a->tls_key_file = acc->tls_key_file ? xstrdup(acc->tls_key_file) : NULL;
 	a->tls_cert_file = 
 	    acc->tls_cert_file ? xstrdup(acc->tls_cert_file) : NULL;
 	a->tls_trust_file = 
 	    acc->tls_trust_file ? xstrdup(acc->tls_trust_file) : NULL;
-	a->tls_nostarttls = acc->tls_nostarttls;
 	a->tls_nocertcheck = acc->tls_nocertcheck;
+	a->tls_force_sslv3 = acc->tls_force_sslv3;
 	a->logfile = acc->logfile ? xstrdup(acc->logfile) : NULL;
 	a->syslog = acc->syslog ? xstrdup(acc->syslog) : NULL;
     }
@@ -462,6 +464,10 @@ void override_account(account_t *acc1, account_t *acc2)
     {
 	acc1->tls = acc2->tls;
     }
+    if (acc2->mask & ACC_TLS_NOSTARTTLS)
+    {
+	acc1->tls_nostarttls = acc2->tls_nostarttls;
+    }
     if (acc2->mask & ACC_TLS_KEY_FILE)
     {
 	free(acc1->tls_key_file);
@@ -480,9 +486,9 @@ void override_account(account_t *acc1, account_t *acc2)
 	acc1->tls_trust_file = 
 	    acc2->tls_trust_file ? xstrdup(acc2->tls_trust_file) : NULL;
     }
-    if (acc2->mask & ACC_TLS_NOSTARTTLS)
+    if (acc2->mask & ACC_TLS_FORCE_SSLV3)
     {
-	acc1->tls_nostarttls = acc2->tls_nostarttls;
+	acc1->tls_force_sslv3 = acc2->tls_force_sslv3;
     }
     if (acc2->mask & ACC_TLS_NOCERTCHECK)
     {
@@ -552,7 +558,7 @@ int check_account(account_t *acc, int sendmail_mode, char **errstr)
     if (acc->tls_nocertcheck && acc->tls_trust_file)
     {
 	*errstr = xasprintf(
-		_("cannot use tls_trust_file with tls_cert_check turned off"));
+		_("cannot use tls_trust_file with tls_certcheck turned off"));
 	return CONF_ESYNTAX;
     }
 
@@ -1094,6 +1100,26 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 		break;
 	    }
 	}
+	else if (strcmp(cmd, "tls_starttls") == 0)
+	{
+	    acc->mask |= ACC_TLS_NOSTARTTLS;
+	    if (*arg == '\0' || is_on(arg))
+	    {
+		acc->tls_nostarttls = 0;
+	    }
+	    else if (is_off(arg))
+	    {
+		acc->tls_nostarttls = 1;
+	    }
+	    else
+	    {
+	    	*errstr = xasprintf(
+    			_("line %d: invalid argument %s for command %s"),
+			line, arg, cmd);
+		e = CONF_ESYNTAX;
+		break;
+	    }
+	}
 	else if (strcmp(cmd, "tls_key_file") == 0)
 	{
 	    acc->mask |= ACC_TLS_KEY_FILE;
@@ -1153,16 +1179,16 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 		break;
 	    }
 	}
-	else if (strcmp(cmd, "tls_starttls") == 0)
+	else if (strcmp(cmd, "tls_force_sslv3") == 0)
 	{
-	    acc->mask |= ACC_TLS_NOSTARTTLS;
+	    acc->mask |= ACC_TLS_FORCE_SSLV3;
 	    if (*arg == '\0' || is_on(arg))
 	    {
-		acc->tls_nostarttls = 0;
+		acc->tls_force_sslv3 = 1;
 	    }
 	    else if (is_off(arg))
 	    {
-		acc->tls_nostarttls = 1;
+		acc->tls_force_sslv3 = 0;
 	    }
 	    else
 	    {
