@@ -3,7 +3,7 @@
  * 
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,46 +23,42 @@
 
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+# include "config.h"
 #endif
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-
-#ifdef __MINGW32__
-#include <winsock2.h>
-#ifdef HAVE_GETADDRINFO
-#include <ws2tcpip.h>
-#endif
-#elif defined (DJGPP)
-#include <unistd.h>
-#include <tcp.h>
-#include <netdb.h>
-#ifndef NI_MAXHOST
-#define NI_MAXHOST 1025
-#endif
-#include <errno.h>
-extern int errno;
-#else /* UNIX */
 #include <unistd.h>
 #include <fcntl.h>
-#ifndef HAVE_GETADDRINFO
-#include <netinet/in.h>
-extern int h_errno;
-#endif
-#include <sys/time.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <arpa/inet.h>
-#include <netdb.h>
 #include <errno.h>
 extern int errno;
-#endif /* UNIX */
+#include <sys/time.h>
+#include <sys/types.h>
+#ifdef W32_NATIVE
+# define WINVER 0x0501
+# include <winsock2.h>
+# include <ws2tcpip.h>
+#else
+# ifndef HAVE_GETADDRINFO
+#  include <netinet/in.h>
+   extern int h_errno;
+# endif
+# include <sys/socket.h>
+# include <arpa/inet.h>
+# include <netdb.h>
+# ifndef NI_MAXHOST
+#  define NI_MAXHOST 1025
+# endif
+#endif
+
+#ifdef HAVE_LIBIDN
+# include <idna.h>
+#endif
 
 #include "gettext.h"
-#include "xvasprintf.h"
 #include "xalloc.h"
+#include "xvasprintf.h"
 
 #include "net.h"
 
@@ -77,7 +73,7 @@ extern int errno;
  * recv(), send()
  */
 
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
 const char *wsa_strerror(int error_code)
 {
     switch (error_code)
@@ -85,6 +81,9 @@ const char *wsa_strerror(int error_code)
 	case WSA_NOT_ENOUGH_MEMORY:
 	    return _("not enough memory");
 	    
+	case WSAEINTR:
+	    return _("operation aborted");
+
 	case WSAEINVAL:
 	    return _("invalid argument");
 	    
@@ -167,7 +166,7 @@ const char *wsa_strerror(int error_code)
     	    return _("unknown error");
     }
 }
-#endif /* __MINGW32__ */
+#endif /* W32_NATIVE */
 
 
 /*
@@ -176,7 +175,7 @@ const char *wsa_strerror(int error_code)
  * see net.h
  */
 
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
 int net_lib_init(char **errstr)
 {
     WORD wVersionRequested;
@@ -212,7 +211,7 @@ int net_lib_init(char **errstr UNUSED)
 
 void net_close_socket(int fd)
 {
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
     (void)closesocket(fd);
 #else
     (void)close(fd);
@@ -233,13 +232,10 @@ void net_close_socket(int fd)
 int net_connect(int fd, const struct sockaddr *serv_addr, socklen_t addrlen,
 	int timeout)
 {
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
     /* TODO: I don't know how to do this on Win32. Please send a patch. */
     return connect(fd, serv_addr, addrlen);
-#elif defined DJGPP
-    /* TODO: I don't know how to do this with Watt32. Please send a patch. */
-    return connect(fd, serv_addr, addrlen);
-#else /* UNIX */
+#else /* UNIX or DJGPP */
     
     int flags;
     struct timeval tv;
@@ -325,7 +321,7 @@ void net_set_io_timeout(int socket, int seconds)
      * en-us/winsock/winsock/setsockopt_2.asp
      * We activate these timeouts only for Windows systems that also have
      * getaddrinfo(), which means XP or newer, to work around this problem. */
-#if defined __MINGW32__ && !defined HAVE_GETADDRINFO
+#if defined W32_NATIVE && !defined HAVE_GETADDRINFO
     /* do nothing */
 #else
     struct timeval tv;
@@ -358,8 +354,12 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
     struct addrinfo *res0;
     struct addrinfo *res;
     int error_code;
+    int saved_errno;
     int cause;
     char nameinfo_buffer[NI_MAXHOST];
+#ifdef HAVE_LIBIDN
+    char *hostname_ascii;
+#endif
     
     hints.ai_family = PF_UNSPEC;
     hints.ai_socktype = SOCK_STREAM;
@@ -370,17 +370,34 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
     hints.ai_addr = NULL;
     hints.ai_next = NULL;
     port_string = xasprintf("%d", port);
+#ifdef HAVE_LIBIDN
+    if (idna_to_ascii_lz(hostname, &hostname_ascii, 0) != IDNA_SUCCESS)
+    {
+	hostname_ascii = xstrdup(hostname);
+    }
+    error_code = getaddrinfo(hostname_ascii, port_string, &hints, &res0);
+    free(hostname_ascii);
+#else
     error_code = getaddrinfo(hostname, port_string, &hints, &res0);
+#endif
     free(port_string);
     if (error_code)
     {
-	*errstr = xasprintf(_("cannot locate host %s: %s"), hostname,
-#ifdef __MINGW32__
-		wsa_strerror(error_code)
+#ifdef W32_NATIVE
+	*errstr = xasprintf(_("cannot locate host %s: %s"), 
+		hostname, wsa_strerror(WSAGetLastError()));
 #else
-		gai_strerror(error_code)
+	if (error_code == EAI_SYSTEM && errno == EINTR)
+	{
+	    *errstr = xasprintf(_("operation aborted"));
+	}
+	else
+	{
+	    *errstr = xasprintf(_("cannot locate host %s: %s"), hostname,
+		    error_code == EAI_SYSTEM ? strerror(errno) 
+		    : gai_strerror(error_code));
+	}
 #endif
-		);
 	return NET_EHOSTNOTFOUND;
     }
 
@@ -397,7 +414,9 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
 	if (net_connect(fd, res->ai_addr, res->ai_addrlen, timeout) < 0)
 	{
 	    cause = 2;
+	    saved_errno = errno;
 	    net_close_socket(fd);
+	    errno = saved_errno;
 	    fd = -1;
 	    continue;
 	}
@@ -439,7 +458,7 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
 	if (cause == 1)
 	{
 	    *errstr = xasprintf(_("cannot create socket: %s"), 
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
 		    wsa_strerror(WSAGetLastError())
 #else
 		    strerror(errno)
@@ -449,14 +468,20 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
 	}
 	else /* cause == 2 */
 	{
+#ifdef W32_NATIVE
 	    *errstr = xasprintf(_("cannot connect to %s, port %d: %s"), 
-		    hostname, port,
-#ifdef __MINGW32__
-		    wsa_strerror(WSAGetLastError())
+		    hostname, port, wsa_strerror(WSAGetLastError()));
 #else
-		    strerror(errno)
+	    if (errno == EINTR)
+	    {
+		*errstr = xasprintf(_("operation aborted"));
+	    }
+	    else
+	    {
+		*errstr = xasprintf(_("cannot connect to %s, port %d: %s"), 
+			hostname, port, strerror(errno));
+	    }
 #endif
-		    );
 	    return NET_ECONNECT;
 	}
     }
@@ -470,13 +495,16 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
     int fd;
     struct sockaddr_in sock;
     struct hostent *remote_host;
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
     unsigned long inaddr;
-#endif /* __MINGW32__ */
+#endif /* W32_NATIVE */
     struct in_addr addr;
     char *p;
+#ifdef HAVE_LIBIDN
+    char *hostname_ascii;
+#endif
     
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
     /* Work around a broken gethostbyname() function on old Windows systems that
      * cannot handle IP addresses. */
     if ((inaddr = inet_addr(hostname)) != INADDR_NONE
@@ -487,11 +515,25 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
 	 * a struct hostent. No need to call gethostbyname() anymore. */
     }
     else
-#endif /* __MINGW32__ */
-    if (!(remote_host = gethostbyname(hostname)))
+    {
+#endif /* W32_NATIVE */
+#ifdef HAVE_LIBIDN
+	if (idna_to_ascii_lz(hostname, &hostname_ascii, 0) != IDNA_SUCCESS)
+	{
+	    hostname_ascii = xstrdup(hostname);
+	}
+	remote_host = gethostbyname(hostname_ascii);
+	free(hostname_ascii);
+#else
+	remote_host = gethostbyname(hostname);
+#endif
+#ifdef W32_NATIVE
+    }
+#endif /* W32_NATIVE */
+    if (!remote_host)
     {
 	*errstr = xasprintf(_("cannot locate host %s: %s"), hostname,
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
 		wsa_strerror(WSAGetLastError())
 #else
 		hstrerror(h_errno)
@@ -503,7 +545,7 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
     if ((fd = socket(PF_INET, SOCK_STREAM, 0)) < 0)
     {
 	*errstr = xasprintf(_("cannot create socket: %s"), 
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
 		wsa_strerror(WSAGetLastError())
 #else
 		strerror(errno)
@@ -519,14 +561,20 @@ int net_open_socket(const char *hostname, int port, int timeout, int *ret_fd,
 
     if (net_connect(fd, (struct sockaddr *)(&sock), sizeof(sock), timeout) < 0)
     {
+#ifdef W32_NATIVE
 	*errstr = xasprintf(_("cannot connect to %s, port %d: %s"),
-		hostname, port,
-#ifdef __MINGW32__
-		wsa_strerror(WSAGetLastError())
+		hostname, port, wsa_strerror(WSAGetLastError()));
 #else
-		strerror(errno)
+	if (errno == EINTR)
+	{
+	    *errstr = xasprintf(_("operation aborted"));
+	}
+	else
+	{
+	    *errstr = xasprintf(_("cannot connect to %s, port %d: %s"),
+		    hostname, port, strerror(errno));
+	}
 #endif
-		);
 	return NET_ECONNECT;
     }
 
@@ -586,7 +634,7 @@ void net_readbuf_init(net_readbuf_t *readbuf)
 int net_readbuf_read(int fd, net_readbuf_t *readbuf, char *ptr, 
 	char **errstr)
 {
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
     
     int e;
     
@@ -618,19 +666,18 @@ int net_readbuf_read(int fd, net_readbuf_t *readbuf, char *ptr,
     *ptr = *((readbuf->ptr)++);
     return 1;
     
-#else /* !__MINGW32__ */
+#else /* !W32_NATIVE */
 
     if (readbuf->count <= 0)
     {
-	do
-	{
-	    readbuf->count = (int)recv(fd, readbuf->buf, 
-		    sizeof(readbuf->buf), 0);
-	}
-	while (readbuf->count < 0 && errno == EINTR);
+    	readbuf->count = (int)recv(fd, readbuf->buf, sizeof(readbuf->buf), 0);
 	if (readbuf->count < 0)
 	{
-	    if (errno == EAGAIN)
+	    if (errno == EINTR)
+	    {
+		*errstr = xasprintf(_("operation aborted"));
+	    }
+	    else if (errno == EAGAIN)
 	    {
 		*errstr = xasprintf(_("network read error: %s"), 
 			_("the operation timed out"));
@@ -652,7 +699,7 @@ int net_readbuf_read(int fd, net_readbuf_t *readbuf, char *ptr,
     *ptr = *((readbuf->ptr)++);
     return 1;
 
-#endif /* !__MINGW32__ */
+#endif /* !W32_NATIVE */
 }
 
 
@@ -703,7 +750,7 @@ int net_gets(int fd, net_readbuf_t *readbuf,
 
 int net_puts(int fd, const char *s, size_t len, char **errstr)
 {
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
 
     int e, ret;
 
@@ -736,7 +783,7 @@ int net_puts(int fd, const char *s, size_t len, char **errstr)
 	return NET_EIO;
     }
 
-#else /* !__MINGW32__ */
+#else /* !W32_NATIVE */
     
     ssize_t ret;
 
@@ -744,14 +791,13 @@ int net_puts(int fd, const char *s, size_t len, char **errstr)
     {
 	return NET_EOK;
     }
-    do
+    if ((ret = send(fd, s, len, 0)) < 0)
     {
-	ret = send(fd, s, len, 0);
-    }
-    while (ret < 0 && errno == EINTR);
-    if (ret < 0)
-    {
-	if (errno == EAGAIN)
+	if (errno == EINTR)
+	{
+	    *errstr = xasprintf(_("operation aborted"));
+	}
+	else if (errno == EAGAIN)
 	{
 	    *errstr = xasprintf(_("network write error: %s"), 
 		    _("the operation timed out"));
@@ -773,7 +819,7 @@ int net_puts(int fd, const char *s, size_t len, char **errstr)
 	return NET_EIO;
     }
 
-#endif /* !__MINGW32__ */
+#endif /* !W32_NATIVE */
 }
 
 
@@ -808,12 +854,14 @@ char *net_get_canonical_hostname(void)
 	hints.ai_canonname = NULL;
 	hints.ai_addr = NULL;
 	hints.ai_next = NULL;
-	if (getaddrinfo(hostname, NULL, &hints, &res0) == 0 
-		&& res0->ai_canonname)
+	if (getaddrinfo(hostname, NULL, &hints, &res0) == 0)
 	{
-	    canonname = xstrdup(res0->ai_canonname);
+	    if (res0->ai_canonname)
+	    {
+		canonname = xstrdup(res0->ai_canonname);
+	    }
+	    freeaddrinfo(res0);
 	}
-	freeaddrinfo(res0);
 #else /* !HAVE_GETADDRINFO */
 	if ((hostent = gethostbyname(hostname)) && hostent->h_name)
 	{
@@ -839,7 +887,7 @@ char *net_get_canonical_hostname(void)
 
 void net_lib_deinit(void)
 {
-#ifdef __MINGW32__
+#ifdef W32_NATIVE
     (void)WSACleanup();
 #endif
 }
