@@ -3,7 +3,7 @@
  * 
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -17,31 +17,27 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- *   msmtp is released under the GPL with the additional exemption that
- *   compiling, linking, and/or using OpenSSL is allowed.
+ *   along with this program; if not, write to the Free Software Foundation,
+ *   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef TLS_H
 #define TLS_H
 
-#ifdef HAVE_GNUTLS
-#include <gnutls/gnutls.h>
-#endif /* HAVE_GNUTLS */
+#ifdef HAVE_LIBGNUTLS
+# include <gnutls/gnutls.h>
+#endif /* HAVE_LIBGNUTLS */
 #ifdef HAVE_OPENSSL
-#include <openssl/ssl.h>
+# include <openssl/ssl.h>
 #endif /* HAVE_OPENSSL */
 
 
-/* All 'errstr' arguments must point to buffers that are at least
- * 'errstr_bufsize' characters long.
+/*
  * If a function with an 'errstr' argument returns a value != TLS_EOK,
- * 'errstr' will contain an error message (maybe just "").
+ * '*errstr' either points to an allocates string containing an error 
+ * description or is NULL.
  * If such a function returns TLS_EOK, 'errstr' will not be changed.
  */
-extern const size_t errstr_bufsize;
 #define TLS_EOK		0	/* no error */
 #define TLS_ELIBFAILED	1	/* The underlying library failed */
 #define TLS_ESEED	2	/* Cannot seed pseudo random number generator */
@@ -60,10 +56,10 @@ typedef struct
 {
     int have_trust_file;
     int is_active;
-#ifdef HAVE_GNUTLS
+#ifdef HAVE_LIBGNUTLS
     gnutls_session_t session;
     gnutls_certificate_credentials_t cred;
-#endif /* HAVE_GNUTLS */
+#endif /* HAVE_LIBGNUTLS */
 #ifdef HAVE_OPENSSL
     SSL_CTX *ssl_ctx;
     SSL *ssl;
@@ -94,10 +90,11 @@ typedef struct
 /*
  * tls_lib_init()
  *
- * Initialize underlying TLS library.
+ * Initialize underlying TLS library. If this function returns TLS_ELIBFAILED,
+ * *errstr will always point to an error string.
  * Used error codes: TLS_ELIBFAILED
  */
-int tls_lib_init(char *errstr);
+int tls_lib_init(char **errstr);
 
 /*
  * tls_clear()
@@ -113,11 +110,14 @@ void tls_clear(tls_t *tls);
  * set to be used when the peer request a certificate. If 'trust_file' is not
  * NULL, it will be used to verify the peer certificate.
  * All files must be in PEM format.
+ * If 'force_sslv3' is set, then only the SSLv3 protocol will be accepted. This
+ * option might be needed to talk to some obsolete broken servers. Only use this
+ * if you have to.
  * Used error codes: TLS_ELIBFAILED, TLS_EFILE
  */
 int tls_init(tls_t *tls, 
 	const char *key_file, const char *ca_file, const char *trust_file, 
-	char *errstr);
+	int force_sslv3, char **errstr);
 
 /*
  * tls_start()
@@ -138,7 +138,7 @@ int tls_init(tls_t *tls,
  * Used error codes: TLS_ELIBFAILED, TLS_ECERT, TLS_EHANDSHAKE
  */
 int tls_start(tls_t *tls, int fd, const char *hostname, int no_certcheck,
-	tls_cert_info_t *tci, char *errstr);
+	tls_cert_info_t *tci, char **errstr);
 
 /*
  * tls_is_active()
@@ -166,7 +166,7 @@ void tls_cert_info_free(tls_cert_info_t *tci);
  * it in 'tci'. See the description of tls_cert_info_t above.
  * Used error codes: TLS_ECERT
  */
-int tls_cert_info_get(tls_t *tls, tls_cert_info_t *tci, char *errstr);
+int tls_cert_info_get(tls_t *tls, tls_cert_info_t *tci, char **errstr);
 
 /*
  * tls_getchar()
@@ -175,7 +175,7 @@ int tls_cert_info_get(tls_t *tls, tls_cert_info_t *tci, char *errstr);
  * Sets the 'eof' flag to 1 if EOF occurs and to 0 otherwise.
  * Used error codes: TLS_EIO
  */
-int tls_getchar(tls_t *tls, char *c, int *eof, char *errstr);
+int tls_getchar(tls_t *tls, char *c, int *eof, char **errstr);
 
 /*
  * tls_gets()
@@ -187,7 +187,7 @@ int tls_getchar(tls_t *tls, char *c, int *eof, char *errstr);
  * number of characters excluding the terminating '\0') will be stored in 'len'.
  * Used error codes: TLS_EIO
  */
-int tls_gets(tls_t *tls, char *str, size_t size, size_t *len, char *errstr);
+int tls_gets(tls_t *tls, char *str, size_t size, size_t *len, char **errstr);
 
 /*
  * tls_puts()
@@ -195,7 +195,7 @@ int tls_gets(tls_t *tls, char *str, size_t size, size_t *len, char *errstr);
  * Writes 'len' characters from the string 's' using TLS.
  * Used error codes: TLS_EIO
  */
-int tls_puts(tls_t *tls, const char *s, size_t len, char *errstr);
+int tls_puts(tls_t *tls, const char *s, size_t len, char **errstr);
 
 /*
  * tls_close()
